import logging

from django.db.models import Model

from isc_common.models.base_ref import BaseRefQuerySet, BaseRefManager

logger = logging.getLogger(__name__)


class BaseQuerySet(BaseRefQuerySet):
    pass


class BaseManager(BaseRefManager):
    def get_queryset(self):
        return BaseQuerySet(model=self.model)


class Base_model(Model):
    description = None
    parent = None
    get_parent = None

    def get_parent_name(self, parent, description):
        if not self.description:
            self.description = ''

        if not parent:
            return description

        item = parent
        description = f'/{item.description}{description}'
        if item.parent:
            return self.get_parent_name(self.get_parent(parent=item.parent), description)
        return description

    @property
    def full_name(self):

        if not self.description:
            self.description = ''

        description = f'/{self.description}'
        return self.get_parent_name(self.get_parent(parent=self.parent), description)

    def __str__(self):
        return f"{self.id}"

    objects = BaseManager()

    class Meta:
        verbose_name = 'Базовый класс для всех объектов 1С'
        abstract = True



import logging

from isc_common.auth.models.user import User
from isc_common.fields.code_field import CodeStrictField
from isc_common.fields.related import ForeignKeyCascade
from isc_common.models.audit import AuditQuerySet, AuditManager, AuditModel
from isc_common.number import DelProps

logger = logging.getLogger(__name__)


class Deleted_progressesQuerySet(AuditQuerySet):
    pass


class Deleted_progressesManager(AuditManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            'editing': record.editing,
            'deliting': record.deliting,
            'props': record.props
        }
        return DelProps(res)

    def get_queryset(self):
        return Deleted_progressesQuerySet(self.model, using=self._db)


class Deleted_progresses(AuditModel):
    id_progress = CodeStrictField()
    user = ForeignKeyCascade(User)

    objects = Deleted_progressesManager()

    def __str__(self):
        return f'ID:{self.id}, id_progress: {self.id_progress}, user: [{self.user}]'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Отражение удаленных запущенных процессов'
        unique_together = (('user', 'id_progress'),)



from django.urls import path

from one_c.views import files_1c

urlpatterns = [

    path('Files_1c/Fetch/', files_1c.Files_1c_Fetch),
    path('Files_1c/Add', files_1c.Files_1c_Add),
    path('Files_1c/Update', files_1c.Files_1c_Update),
    path('Files_1c/Remove', files_1c.Files_1c_Remove),
    path('Files_1c/Lookup/', files_1c.Files_1c_Lookup),
    path('Files_1c/Info/', files_1c.Files_1c_Info),

]



# Generated by Django 2.2.5 on 2019-09-16 17:59

from django.db import migrations
import isc_common.fields.description_field


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0064_auto_20190909_1409'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages',
            name='message',
            field=isc_common.fields.description_field.DescriptionField(),
        ),
    ]



# Generated by Django 2.2.7 on 2019-11-16 10:36

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('history', '0004_auto_20191017_1135'),
    ]

    operations = [
        migrations.CreateModel(
            name='Visitor',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('session_key', models.CharField(max_length=40)),
                ('ip_address', models.CharField(max_length=20)),
                ('username', models.CharField(blank=True, max_length=255, null=True)),
                ('user_agent', models.CharField(max_length=255)),
                ('referrer', models.CharField(max_length=255)),
                ('url', models.CharField(max_length=255)),
                ('session_start', models.DateTimeField()),
                ('last_update', models.DateTimeField()),
            ],
            options={
                'verbose_name': 'Посетитель',
                'ordering': ('-last_update',),
                'unique_together': {('session_key', 'ip_address')},
            },
        ),
    ]



from django.urls import path

from isc_common.views import progresses

urlpatterns = [

    path('Progresses/Fetch/', progresses.Progresses_Fetch),
    path('Progresses/Add', progresses.Progresses_Add),
    path('Progresses/Update', progresses.Progresses_Update),
    path('Progresses/Remove', progresses.Progresses_Remove),
    path('Progresses/Lookup/', progresses.Progresses_Lookup),
    path('Progresses/Info/', progresses.Progresses_Info),
    path('Progresses/Copy', progresses.Progresses_Copy),
    path('Progresses/Resore/', progresses.Progresses_Resore),

]



import datetime
import json
import logging
import os
import uuid

from django.db.models.query import QuerySet

from isc_common import setAttr
from isc_common.common import uuid4
from list.listCalendarEvent import LinkedList

logger = logging.getLogger(__name__)


class CalendarEvent:
    backgroundColor = None
    borderColor = 'black'
    canDrag = False
    canEdit = False
    canEditLane = False
    canEditSublane = False
    canResize = False
    description = None
    duration = None
    durationUnit = "hour"
    endDate = None
    headerBackgroundColor = None
    headerBorderColor = None
    headerTextColor = None
    id = None
    isholiday = None
    isworkday = None
    isredlabelday = None
    length = None
    lane = None
    name = None
    startDate = None
    styleName = None
    sublane = None
    textColor = None

    def __init__(self, *args, **kwargs):
        for k, v in kwargs.items():
            if isinstance(v, bool):
                setattr(self, k, 1 if v == True else 0)
            elif isinstance(v, datetime.datetime):
                if isinstance(v, datetime.datetime):
                    r = v.isoformat()
                    if v.microsecond:
                        r = r[:23] + r[26:]
                    if r.endswith('+00:00'):
                        r = r[:-6] + 'Z'
                setattr(self, k, r)
            else:
                setattr(self, k, v() if callable(v) else v)

    def __str__(self):
        setAttr(self.__dict__, "startDate", str(self.__dict__.get('startDate')))
        setAttr(self.__dict__, "endDate", str(self.__dict__.get('endDate')))
        return f'{str(self.__dict__)}'

    def print(self, comment=''):
        print('\n')
        print(f'{comment}({self.__hash__()}) {str(self)}')

    def copy(self, *args, **kwargs):
        res = CalendarEvent(**self.__dict__)
        for k, v in kwargs.items():
            setattr(res, k, v() if callable(v) else v)
        return res

    def to_json(self):
        return self.__dict__


class CalendarEventLinkedList(LinkedList):
    def __init__(self, **kwargs):
        super().__init__(**kwargs)
        if kwargs.get('query'):
            query = kwargs.get('query')
            if isinstance(kwargs.get('query'), QuerySet):
                for item in query:
                    self.add(item)

    def fusion(self, fusion_list):
        if not isinstance(fusion_list, CalendarEventLinkedList):
            raise Exception(f'ce is not CalendarEventLinkedList instance')

        relax = fusion_list.first
        while relax:
            shift = self.first
            y = 0
            while shift:
                # shift.item.print('shift')
                # relax.item.print('relax')
                if shift.item.isworkday and shift.item.startDate < relax.item.startDate:
                    if shift.item.endDate > relax.item.startDate:
                        if shift.item.endDate > relax.item.endDate:
                            # self.print('self:')
                            self.replace(y, shift.item.copy(id=uuid4(), endDate=relax.item.startDate))
                            # self.print('self:')
                            self.insert(y + 1, relax.item.copy(id=uuid4(), ))
                            # self.print('self:')
                            self.insert(y + 2, shift.item.copy(id=uuid4(), startDate=relax.item.endDate))
                            # self.print('self:')
                            break
                        else:
                            raise Exception(f'Unknown case.')
                    else:
                        pass
                else:
                    pass
                shift = shift.next
                y += 1
            relax = relax.next

    def to_json(self):
        res = []

        if self.first is not None:
            current = self.first
            res.append(current.item.to_json())

            while current.next is not None:
                current = current.next
                res.append(current.item.to_json())

        return res

    def write_2_file(self, filename):
        dir, _ = os.path.split(filename)
        dir = f'{os.path.curdir}{os.sep}{dir}'
        if not os.path.exists(dir):
            os.makedirs(dir, exist_ok=True)

        outF = open(filename, "w")

        def pritnt(str):
            # logger.debug(str)
            outF.write(str)

        if self.first is not None:

            current = self.first
            pritnt('simpleSyS.tag=')
            pritnt('[')

            pritnt(f'{json.dumps(current.item.to_json())},')
            pritnt("\n")

            while current.next is not None:
                current = current.next
                pritnt(f'{json.dumps(current.item.to_json())},')
                pritnt("\n")
            pritnt(']')

        outF.close()
        print('Запись выполнена.')



from clndr.models.holidays import Holidays, HolidaysManager
from isc_common.http.DSResponse import DSResponseUpdate, DSResponseAdd, DSResponse, JsonResponseWithException
from isc_common.http.RPCResponse import RPCResponseConstant
from isc_common.http.response import JsonResponse


@JsonResponseWithException()
def Holidays_Fetch(request):
    return JsonResponse(
        DSResponse(
            request=request,
            data=Holidays.objects.
                select_related('daytype').
                get_range_rows1(
                request=request,
                function=HolidaysManager.getRecord
            ),
            status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Holidays_Add(request):
    return JsonResponse(DSResponseAdd(data=Holidays.objects.createFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Holidays_Update(request):
    return JsonResponse(DSResponseUpdate(data=Holidays.objects.updateFromRequest(request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Holidays_Remove(request):
    return JsonResponse(DSResponse(request=request, data=Holidays.objects.deleteFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Holidays_Lookup(request):
    return JsonResponse(DSResponse(request=request, data=Holidays.objects.lookupFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Holidays_Info(request):
    return JsonResponse(DSResponse(request=request, data=Holidays.objects.get_queryset().get_info(request=request), status=RPCResponseConstant.statusSuccess).response)


@JsonResponseWithException()
def Holidays_Copy(request):
    return JsonResponse(DSResponse(request=request, data=Holidays.objects.copyFromRequest(request=request), status=RPCResponseConstant.statusSuccess).response)



from isc_common import Stack
from isc_common.isc.data_binding.advanced_criteria import AdvancedCriteria
from isc_common.isc.data_binding.criterion import Criterion


class CriteriaStack(Stack):
    def __init__(self, request_criteria):
        self.stack = []

        if isinstance(request_criteria, list):
            self.extend([AdvancedCriteria(**criterion) if criterion.get('_constructor') == 'AdvancedCriteria' else Criterion(**criterion) for criterion in request_criteria])
        elif isinstance(request_criteria, dict):
            self.push(AdvancedCriteria(**request_criteria) if request_criteria.get('_constructor') else Criterion(**request_criteria))



from django.urls import path

from clndr.views import day

urlpatterns = [

    path('Day/Fetch/', day.Day_Fetch),
    path('Day/Add', day.Day_Add),
    path('Day/Update', day.Day_Update),
    path('Day/Remove', day.Day_Remove),
    path('Day/Lookup/', day.Day_Lookup),
    path('Day/Info/', day.Day_Info),
    path('Day/Copy', day.Day_Copy),

]



from transliterate import translit
from transliterate.exceptions import LanguageDetectionError

from isc_common import setAttr


class EntityProperties:
    code = None

    def used_field(self):
        return []

    def __init__(self):
        from one_c.models.entity_1c import Entity_1c
        if self.code:
            self.id = Entity_1c.objects.get(code=self.code).id


class ForeignKey:
    def __init__(self, **kwargs):
        self.type = kwargs.get('type')
        self.to = kwargs.get('to', "'self'")
        self.pk = kwargs.get('pk', "id")

    def __str__(self):
        return f'{self.type}({self.to})'


class ForeignKeyProtect(ForeignKey):
    def __init__(self, **kwargs):
        setAttr(kwargs, 'type', 'ForeignKeyProtect')
        super().__init__(**kwargs)


class Field:
    foreign_key = None

    @classmethod
    def string(cls):
        return "string"

    @classmethod
    def float(cls):
        return "float"

    @classmethod
    def int(cls):
        return "int"

    @classmethod
    def uuid(cls):
        return "uuid"

    @classmethod
    def boolean(cls):
        return "boolean"

    @property
    def translited_name(self):
        try:
            return translit(self.name, reversed=True).replace("'", '').replace("'", '')
        except LanguageDetectionError:
            return self.name

    @property
    def translited_record_path(self):
        if self.foreign_key:
            path = f'{self.name}.{self.foreign_key.pk}'
        else:
            path = self.name
        try:
            return translit(path, reversed=True).replace("'", '').replace("'", '')
        except LanguageDetectionError:
            return path

    @property
    def translited_alias(self):
        try:
            return translit(self.alias, reversed=True).replace("'", '').replace("'", '')
        except LanguageDetectionError:
            return self.alias

    def __init__(self, **kwargs):
        self.name = kwargs.get('name')
        self.alias = kwargs.get('alias', self.name)
        self.title = kwargs.get('title', self.name)
        self.type = kwargs.get('type', Field.string())
        self.primary_key = kwargs.get('primary_key', False)
        self.required = kwargs.get('required', False)
        self.length = kwargs.get('length', 255),
        self.hidden = kwargs.get('hidden', False)
        self.canEdit = kwargs.get('canEdit', True)
        self.canFilter = kwargs.get('canFilter', True)
        self.align = kwargs.get('align', '"left"')
        self.canSort = kwargs.get('canSort', False)
        self.format = kwargs.get('format', 'undefined')
        self.canGroupBy = kwargs.get('canGroupBy', False)
        self.index = kwargs.get('index', 0)
        self.mask = kwargs.get('mask', '""')
        self.lookup = kwargs.get('lookup', False)
        self.only_datasource = kwargs.get('only_datasource', False)
        if kwargs.get('foreign_key'):
            if isinstance(kwargs.get('foreign_key'), ForeignKey):
                self.foreign_key = kwargs.get('foreign_key')
            else:
                raise Exception(f"{kwargs.get('foreign_key')} must be a ForeignKey instance")


class Entities:

    def __init__(self, entities=[]):
        self.entities = entities

    def get_field(self, id, name):
        _entity = [_entity for _entity in self.entities if _entity.id == id]
        if len(_entity) == 0:
            return None
        else:
            fields = [field for field in _entity[0].used_field() if field.name == name]
            if len(fields) == 0:
                return None
            else:
                return fields[0]

    def get_fields(self, id):
        _entity = [_entity for _entity in self.entities if _entity.id == id]
        if len(_entity) == 0:
            return None
        else:
            return [field for field in _entity[0].used_field()]



import logging

from django.db.models import Manager, TextField
from isc_common.models.audit import AuditModel

logger = logging.getLogger(__name__)


class Files_group(AuditModel):
    code = TextField(db_index=True)

    objects = Manager()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Группы файлов'



# Generated by Django 2.2.5 on 2019-09-02 18:06

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0040_messages_theme_view'),
    ]

    operations = [
        migrations.AddField(
            model_name='messages',
            name='guid',
            field=models.UUIDField(blank=True, null=True),
        ),
    ]



# Generated by Django 2.2.3 on 2019-07-06 20:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('one_c', '0012_auto_20190706_1121'),
    ]

    operations = [
        migrations.CreateModel(
            name='Uuid_2_id',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', models.UUIDField(db_index=True)),
            ],
            options={
                'verbose_name': 'Таблица соответствия ID ',
            },
        ),
        migrations.AlterUniqueTogether(
            name='documents_param_1c',
            unique_together={('type', 'value', 'value_boolean', 'value_float', 'value_int', 'value_uuid')},
        ),
    ]



from django.db.models import BinaryField, TextField

from isc_common.fields.code_field import CodeField
from isc_common.models.audit import AuditModel


class DbStorage(AuditModel):
    blob = BinaryField()
    filename = TextField(unique=True)
    mimetype = CodeField()

    class Meta:
        abstract = True



# Generated by Django 3.0.3 on 2020-02-10 18:42

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0046_auto_20200210_1834'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='progresses',
            name='channel',
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-23 03:52

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0034_auto_20190823_0350'),
    ]

    operations = [
        migrations.DeleteModel(
            name='Calendar_shifts_days',
        ),
    ]



# Generated by Django 3.1.5 on 2021-01-21 06:33

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0057_debug'),
    ]

    operations = [
        migrations.AddField(
            model_name='debug',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AddField(
            model_name='debug',
            name='deliting',
            field=models.BooleanField(default=True, verbose_name='Возможность удаления'),
        ),
        migrations.AddField(
            model_name='debug',
            name='editing',
            field=models.BooleanField(default=True, verbose_name='Возможность редактирования'),
        ),
        migrations.AddField(
            model_name='debug',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='debug',
            name='id',
            field=models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор'),
        ),
    ]



from django.urls import path

from one_c.views import param_type

urlpatterns = [

    path('Param_type/Fetch/', param_type.Param_type_Fetch),
    path('Param_type/Add', param_type.Param_type_Add),
    path('Param_type/Update', param_type.Param_type_Update),
    path('Param_type/Remove', param_type.Param_type_Remove),
    path('Param_type/Lookup/', param_type.Param_type_Lookup),
    path('Param_type/Info/', param_type.Param_type_Info),

]



import calendar
import unittest
from datetime import datetime

from clndr.calendar_event import CalendarEvent, CalendarEventLinkedList


class FusionChainTestCase(unittest.TestCase):

    def setUp(self) -> None:
        self.shifts = CalendarEventLinkedList()
        self.relaxes = CalendarEventLinkedList()
        # self.red_letter_days = CalendarEventLinkedList()
        #
        # self.red_letter_days.add(
        #     CalendarEvent(
        #         startDate=datetime(year=2019, month=1, day=1, hour=0, minute=0, second=0),
        #         endDate=datetime(year=2019, month=1, day=9, hour=0, minute=0, second=0),
        #     )
        # )
        #
        # self.red_letter_days.add(
        #     CalendarEvent(
        #         startDate=datetime(year=2019, month=2, day=23, hour=0, minute=0, second=0),
        #         endDate=datetime(year=2019, month=2, day=24, hour=0, minute=0, second=0),
        #     )
        # )
        #
        # self.red_letter_days.add(
        #     CalendarEvent(
        #         startDate=datetime(year=2019, month=3, day=8, hour=0, minute=0, second=0),
        #         endDate=datetime(year=2019, month=3, day=9, hour=0, minute=0, second=0),
        #     )
        # )
        #
        # self.red_letter_days.add(
        #     CalendarEvent(
        #         startDate=datetime(year=2019, month=5, day=1, hour=0, minute=0, second=0),
        #         endDate=datetime(year=2019, month=5, day=4, hour=0, minute=0, second=0),
        #     )
        # )
        #
        # self.red_letter_days.add(
        #     CalendarEvent(
        #         startDate=datetime(year=2019, month=5, day=9, hour=0, minute=0, second=0),
        #         endDate=datetime(year=2019, month=5, day=11, hour=0, minute=0, second=0),
        #     )
        # )
        #
        # self.red_letter_days.add(
        #     CalendarEvent(
        #         startDate=datetime(year=2019, month=6, day=12, hour=0, minute=0, second=0),
        #         endDate=datetime(year=2019, month=6, day=13, hour=0, minute=0, second=0),
        #     )
        # )
        #
        # self.red_letter_days.add(
        #     CalendarEvent(
        #         startDate=datetime(year=2019, month=11, day=4, hour=0, minute=0, second=0),
        #         endDate=datetime(year=2019, month=11, day=5, hour=0, minute=0, second=0),
        #     )
        # )

    def test_одного_рабочего_дня_2_смены_со_всеми_отдыхами_и_перекурами(self):
        self.shifts.add(CalendarEvent(
            startDate=datetime(year=2019, month=1, day=1, hour=7),
            endDate=datetime(year=2019, month=1, day=1, hour=16),
        ))

        self.shifts.add(CalendarEvent(
            startDate=datetime(year=2019, month=1, day=1, hour=16),
            endDate=datetime(year=2019, month=1, day=2, hour=1),
        ))

        self.relaxes.add(CalendarEvent(
            startDate=datetime(year=2019, month=1, day=1, hour=9),
            endDate=datetime(year=2019, month=1, day=1, hour=9, minute=15),
        ))

        self.relaxes.add(CalendarEvent(
            startDate=datetime(year=2019, month=1, day=1, hour=11),
            endDate=datetime(year=2019, month=1, day=1, hour=12),
        ))

        self.relaxes.add(CalendarEvent(
            startDate=datetime(year=2019, month=1, day=1, hour=14),
            endDate=datetime(year=2019, month=1, day=1, hour=14, minute=15),
        ))

        self.relaxes.add(CalendarEvent(
            startDate=datetime(year=2019, month=1, day=1, hour=18),
            endDate=datetime(year=2019, month=1, day=1, hour=18, minute=15),

        ))

        self.relaxes.add(CalendarEvent(
            startDate=datetime(year=2019, month=1, day=1, hour=20),
            endDate=datetime(year=2019, month=1, day=1, hour=21),

        ))

        self.relaxes.add(CalendarEvent(
            startDate=datetime(year=2019, month=1, day=1, hour=23),
            endDate=datetime(year=2019, month=1, day=1, hour=23, minute=15),

        ))

        # self.shifts.print('shifts')
        # self.relaxes.print('relaxes')

        self.shifts.fusion(self.relaxes)
        self.shifts.print('shifts')


if __name__ == '__main__':
    unittest.main()



from django.urls import path

from tracker.views import messages_view

urlpatterns = [

    path('Messages_view/Fetch/', messages_view.Messages_view_Fetch),
    path('Messages_view/FetchAdmin/', messages_view.Messages_view_FetchAdmin),
    path('Messages_view/Add', messages_view.Messages_view_Add),
    path('Messages_view/Update', messages_view.Messages_view_Update),
    path('Messages_view/Remove', messages_view.Messages_view_Remove),
    path('Messages_view/Lookup/', messages_view.Messages_view_Lookup),
    path('Messages_view/Info/', messages_view.Messages_view_Info),

]



import logging

from django.core.management import BaseCommand
from django.db import connection

from code_generator.core.writer import dbl_qutes_str
from isc_common.common.mat_views import exists_table

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    def add_arguments(self, parser):
        parser.add_argument('--obj_name', type=str)
        parser.add_argument('--prefix', type=str)

    def handle(self, *args, **options):
        # obj_name = options.get('obj_name')
        # prefix = options.get('prefix', None)
        #
        # sql_text = '''SELECT a.attname
        #                 --        pg_catalog.format_type(a.atttypid, a.atttypmod),
        #                 --        a.attnotnull,
        #                 --        a.atthasdef,
        #                 --        a.attnum
        #                 FROM pg_catalog.pg_attribute a
        #                 WHERE a.attrelid = (
        #                     SELECT c.oid
        #                     FROM pg_catalog.pg_class c --Этот подзапрос возвращает --идентификатор таблицы...
        #                              LEFT JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
        #                     WHERE pg_catalog.pg_table_is_visible(c.oid)
        #                       AND c.relname ~ %s
        #                 )
        #                   AND a.attnum > 0
        #                   AND NOT a.attisdropped
        #                 ORDER BY a.attnum
        #                 '''
        #
        # sql_text = '''SELECT 1
        #                 FROM   pg_catalog.pg_class c
        #                 JOIN   pg_catalog.pg_namespace n ON n.oid = c.relnamespace
        #                 WHERE  n.nspname = 'schema_name'
        #                 AND    c.relname = 'table_name'
        #                 AND    c.relkind = 'r'''
        #
        # with connection.cursor() as cursor:
        #     cursor.execute(sql_text, [obj_name])
        #     rows = cursor.fetchall()
        #     columns = []
        #     for row in rows:
        #         column_name, = row
        #         if prefix:
        #             columns.append(f'{prefix}.{dbl_qutes_str(column_name)}')
        #         else:
        #             columns.append(dbl_qutes_str(column_name))
        #
        # res = ',\n'.join(columns)
        print(exists_table('ckk_material_askon_mview'))



# Generated by Django 2.2.6 on 2019-10-30 21:04

import bitfield.models
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models

import isc_common.fields.code_field
import isc_common.fields.description_field
import isc_common.fields.name_field
import isc_common.fields.related


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Event_types',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('name', isc_common.fields.name_field.NameField(blank=True, null=True)),
                ('description', isc_common.fields.description_field.DescriptionField()),
                ('code', isc_common.fields.code_field.CodeStrictField(unique=True)),
                ('props', bitfield.models.BitField((('isEvent', 'Является событием', 'compulsory_reading', 'Обязательное прочтение'),), db_index=True, default=0)),
                ('parent', isc_common.fields.related.ForeignKeyProtect(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='events.Event_types')),
            ],
            options={
                'verbose_name': 'Типы событий',
            },
        ),
        migrations.CreateModel(
            name='Event_type_users',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('event_type', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='events.Event_types')),
                ('user', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Подписки на события',
                'unique_together': {('event_type', 'user')},
            },
        ),
    ]



from itertools import groupby

from django.db import transaction

from isc_common import delAttr, setAttr
from isc_common.models.audit import AuditManager, AuditQuerySet


class CommonManagetWithLookUpFieldsQuerySet(AuditQuerySet):
    def create(self, **kwargs):
        setAttr(kwargs, '_operation', 'create')
        return self._create_or_update(**kwargs)

    def update(self, **kwargs):
        setAttr(kwargs, '_operation', 'update')
        return self._create_or_update(**kwargs)

    @transaction.atomic
    def _create_or_update(self, **kwargs):
        def get_objects_data(**data):
            for key in data:
                key1 = None
                key2 = key

                if key.find('__') != -1:
                    key1 = key[0:key.find('__')]
                    key2 = key[key.find('__') + 2:]
                elif key.endswith('_id'):
                    key1 = key[0:key.rfind('_')]
                    key2 = key[key.rfind('_') + 1:]

                yield (((key1, key2), data[key]))

        data_for_grouping = get_objects_data(**kwargs)

        data = dict()
        operation = kwargs.get('_operation')

        for key, group in groupby(data_for_grouping, lambda x: x[0][0]):
            if not data.get(key):
                if key:
                    forignFld = self.get_field(key)
                    group1 = [x for x in group]
                    id = self.get_id_tuple(group1)
                    try:
                        if forignFld:
                            if id is None:
                                data[key] = None
                            else:
                                if isinstance(id, list):
                                    id = id[0]
                                obj = forignFld.related_model.objects.get(pk=id)
                                data[key] = obj

                    except forignFld.related_model.DoesNotExist:
                        data1 = dict()
                        for x in group1:
                            key1 = x[0][1]
                            value = x[1]
                            data1[key1] = value
                        try:
                            if forignFld:
                                obj = forignFld.related_model.objects.get(**data1)
                                data[key] = obj
                        except forignFld.related_model.DoesNotExist:
                            pass
                        except forignFld.related_model.MultipleObjectsReturned:
                            for obj in forignFld.related_model.objects.filter(**data1):
                                data[key] = obj
                                break

                else:
                    group1 = [x for x in group]
                    id = self.get_id_tuple(group1)
                    # if id:
                    #     operation = "update"
                    for x in group1:
                        key = x[0][1]
                        value = x[1]
                        if not data.get(key):
                            data[key] = value

        delAttr(data, 'isFolder')
        delAttr(data, '_operation')
        if operation == 'create':
            return super().create(**data)
        else:
            return super().update(**data)


class CommonManagetWithLookUpFieldsManager(AuditManager):

    def merge_two_dicts(self, x, y):
        z = x.copy()  # start with x's keys and values
        z.update(y)  # modifies z with y's keys and values & returns None
        return z

    def get_queryset(self):
        return CommonManagetWithLookUpFieldsQuerySet(self.model, using=self._db)



import logging

from django.forms import model_to_dict

from isc_common import setAttr, delAttr, dictinct_list
from isc_common.auth.managers.base_user_manager import BaseUserManager
from isc_common.auth.models.usergroup import UserGroup
from isc_common.bit import IsBitOn
from isc_common.http.DSRequest import DSRequest

logger = logging.getLogger(__name__)


class UserManager(BaseUserManager):
    use_in_migrations = True

    @classmethod
    def getRecord(cls, **kwargs):
        record = kwargs.get('record')
        res = {
            "id": record.id,
            "username": record.username,
            "description": record.description,
            "first_name": record.first_name,
            "last_name": record.last_name,
            "email": record.email,
            "middle_name": record.middle_name,
            "short_name": record.get_short_name,
            "short_name1": record.get_short_name1,
            "password": record.password,
            "last_login": record.last_login,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
            "color": record.color,
            "bot": IsBitOn(record.props, 0),
        }
        return res

    @classmethod
    def getRecord1(cls, record):
        res = {
            "id": record.id,
            "username": record.username,
            "description": record.description,
            "first_name": record.first_name,
            "last_name": record.last_name,
            "email": record.email,
            "middle_name": record.middle_name,
            "short_name": record.get_short_name,
            "short_name1": record.get_short_name1,
            "password": record.password,
            "last_login": record.last_login,
            "lastmodified": record.lastmodified,
            "editing": record.editing,
            "deliting": record.deliting,
            "color": record.color,
            "bot": IsBitOn(record.props, 0),
        }
        return res


    def updateFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        delAttr(data, 'usergroup')
        oldValues = request.get_oldValues()

        _oldValues = oldValues.get( 'data')
        if not _oldValues:
            _oldValues = oldValues

        data = self._remove_prop_(data)
        values = [item for item in dictinct_list(_oldValues) - set(data) if not item.startswith('_')]
        for item in values:
            setAttr(data, item, None)

        user_id = request.get_id()

        delAttr(data, 'short_name1')
        delAttr(data, 'status')
        delAttr(data, 'timer')
        delAttr(data, 'timer4Close')
        delAttr(data, 'ws')
        delAttr(data, 'grid')
        delAttr(data, 'short_name')
        delAttr(data, 'bot')

        super().filter(id=user_id).update(**data)

        if data.get('password', None) != _oldValues.get( 'password'):
            user = self.model.objects.get(pk=user_id)
            user.set_password(data.get('password', None))
            user.save()
            user = self.model.objects.get(pk=user_id)
            setAttr(data, 'password', user.password)
            setAttr(data, 'short_name', user.get_short_name)

        return data

    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data(excluded_keys=['id'])
        usergroup_id = data.get('usergroup_id')
        delAttr(data, 'usergroup_id')
        delAttr(data, 'on_line')
        user = super().create(**self.clone_data(data))
        password = request.get_data(excluded_keys=['id']).get('password', None)
        user = self.model.objects.get(pk=user.id)
        user.set_password(password)
        user.save(using=self._db)
        user = self.model.objects.get(pk=user.id)
        if isinstance(usergroup_id, list):
            for group in usergroup_id:
                user.usergroup.add(group)
        res = model_to_dict(user)
        setAttr(res, 'props', res.get('props')._value)
        return self.clone_data(res)

    def _create_user(self, usergroup, username, email, password, **extra_fields):
        """
        Create and save a user with the given username, email, and password.
        """
        if not username:
            raise ValueError('The given username must be set')

        email = self.normalize_email(email)
        username = self.model.normalize_username(username)
        user = self.model(username=username, email=email, **extra_fields)
        user.set_password(password)
        user.save(using=self._db)

        if isinstance(usergroup, list):
            user.usergroup.set(usergroup)
        elif isinstance(usergroup, UserGroup):
            user.usergroup.set([usergroup])

        return user

    def create_user(self, usergroup, username, email=None, password=None, **extra_fields):
        try:
            return self.model.objects.get(username=username)
        except self.model.DoesNotExist:
            return self._create_user(usergroup, username, email, password, **extra_fields)

    def create_superuser(self, username, email, password, **extra_fields):
        try:
            return self.model.objects.get(username=username)
        except self.model.DoesNotExist:
            return self._create_user(usergroup=UserGroup.objects.get(code='administrators'), username=username, email=email, password=password, **extra_fields)



import logging

from clndr.models.calendars import Calendars
from clndr.models.day_types import Day_types
from isc_common.fields.date_time_field import DateTimeField
from isc_common.fields.related import ForeignKeyProtect, ForeignKeyCascade
from isc_common.managers.common_managet_with_lookup_fields import CommonManagetWithLookUpFieldsManager, CommonManagetWithLookUpFieldsQuerySet
from isc_common.models.base_ref import BaseRef

logger = logging.getLogger(__name__)


class HolidaysQuerySet(CommonManagetWithLookUpFieldsQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class HolidaysManager(CommonManagetWithLookUpFieldsManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            'code': record.code,
            'name': record.name,
            'begindate': record.begindate,
            'enddate': record.enddate,
            'description': record.description,
            'calendar_id': record.calendar.id,
            'daytype_id': record.daytype.id,
            'color': record.daytype.color,
            'daytype__code': record.daytype.code,
            'daytype__name': record.daytype.name,
            'daytype__length': record.daytype.length,
        }
        return res

    def get_queryset(self):
        return HolidaysQuerySet(self.model, using=self._db)


class Holidays(BaseRef):
    calendar = ForeignKeyCascade(Calendars)
    daytype = ForeignKeyProtect(Day_types)
    begindate = DateTimeField(verbose_name='Начало', db_index=True)
    enddate = DateTimeField(verbose_name='Конец', db_index=True)

    objects = HolidaysManager()

    def __str__(self):
        return f"ID={self.id}, " \
               f"code={self.code}, " \
               f"name={self.name}, " \
               f"description={self.description}, " \
               f"daytype=[{self.daytype}], " \
               f"begintime={self.begindate}, " \
               f"endtime={self.enddate}"

    class Meta:
        verbose_name = 'Праздничные дни'



from django.urls import path

from clndr.views import shift_day

urlpatterns = [

    path('Shift_day/Fetch/', shift_day.Shift_day_Fetch),
    path('Shift_day/Add', shift_day.Shift_day_Add),
    path('Shift_day/Update', shift_day.Shift_day_Update),
    path('Shift_day/Remove', shift_day.Shift_day_Remove),
    path('Shift_day/Lookup/', shift_day.Shift_day_Lookup),
    path('Shift_day/Info/', shift_day.Shift_day_Info),
    path('Shift_day/Copy', shift_day.Shift_day_Copy),

]



from django.urls import path

from refs.views import type_param_values

urlpatterns = [

    path('Type_param_values/Fetch/', type_param_values.Type_param_values_Fetch),
    path('Type_param_values/Add', type_param_values.Type_param_values_Add),
    path('Type_param_values/Update', type_param_values.Type_param_values_Update),
    path('Type_param_values/Remove', type_param_values.Type_param_values_Remove),
    path('Type_param_values/Lookup/', type_param_values.Type_param_values_Lookup),
    path('Type_param_values/Info/', type_param_values.Type_param_values_Info),
    path('Type_param_values/Copy', type_param_values.Type_param_values_Copy),

]



# Generated by Django 2.2 on 2019-04-14 09:23

from django.db import migrations, models
import django.utils.timezone


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0014_auto_20190414_0920'),
    ]

    operations = [
        migrations.AlterField(
            model_name='messages',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='messages',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='messages_state',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='messages_state',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
        migrations.AlterField(
            model_name='messages_theme',
            name='deleted_at',
            field=models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления'),
        ),
        migrations.AlterField(
            model_name='messages_theme',
            name='lastmodified',
            field=models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление'),
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-17 17:58

from django.conf import settings
from django.db import migrations, models
import django.db.models.deletion
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('tracker', '0020_auto_20190516_0648'),
    ]

    operations = [
        migrations.CreateModel(
            name='Messages_whom',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('message', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to='tracker.Messages')),
                ('to_whom', isc_common.fields.related.ForeignKeyCascade(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'verbose_name': 'Получатель сообщения',
            },
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-23 09:29

from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0036_auto_20190823_0357'),
    ]

    operations = [
        migrations.AlterField(
            model_name='shift_day',
            name='calendar',
            field=isc_common.fields.related.ForeignKeyProtect(default=None, on_delete=django.db.models.deletion.PROTECT, to='clndr.Calendars'),
        ),
        migrations.CreateModel(
            name='Calendar_shifts_days',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('calendar', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='clndr.Calendars')),
                ('friday', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='friday_rel', to='clndr.Shift_day')),
                ('monday', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='monday_rel', to='clndr.Shift_day')),
                ('saturday', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='saturday_rel', to='clndr.Shift_day')),
                ('shift', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='clndr.Shifts')),
                ('sunday', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='sunday_rel', to='clndr.Shift_day')),
                ('thursday', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='thursday_rel', to='clndr.Shift_day')),
                ('tuesday', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='tuesday_rel', to='clndr.Shift_day')),
                ('wednesday', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, related_name='wednesday_rel', to='clndr.Shift_day')),
            ],
            options={
                'verbose_name': 'Рабочие смены по дням',
            },
        ),
    ]



# Generated by Django 2.2.4 on 2019-08-21 01:08

import bitfield.models
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import isc_common.fields.related


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0006_auto_20190821_0100'),
    ]

    operations = [
        migrations.CreateModel(
            name='Shift_day',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('begintime', models.TimeField(db_index=True, verbose_name='Начало')),
                ('endtime', models.TimeField(db_index=True, verbose_name='Конец')),
                ('props', bitfield.models.BitField((('work', 'Рабочий'), ('holiday', 'Выходной')), db_index=True, default=None)),
                ('day', isc_common.fields.related.ForeignKeyProtect(on_delete=django.db.models.deletion.PROTECT, to='clndr.Day')),
            ],
            options={
                'verbose_name': 'День смены',
            },
        ),
    ]



# Generated by Django 2.2.1 on 2019-05-15 19:59

from django.db import migrations, models
import django.utils.timezone
import isc_common.fields.code_field
import isc_common.fields.description_field


class Migration(migrations.Migration):

    dependencies = [
        ('tracker', '0018_delete_messages_view'),
    ]

    operations = [
        migrations.CreateModel(
            name='Messages_view',
            fields=[
                ('id', models.BigAutoField(primary_key=True, serialize=False, verbose_name='Идентификатор')),
                ('id_old', models.BigIntegerField(blank=True, null=True, verbose_name='Идентификатор старый')),
                ('deleted_at', models.DateTimeField(blank=True, db_index=True, null=True, verbose_name='Дата мягкого удаления')),
                ('editing', models.BooleanField(default=True, verbose_name='Возможность редактирования')),
                ('deliting', models.BooleanField(default=True, verbose_name='Возможность удаления')),
                ('lastmodified', models.DateTimeField(db_index=True, default=django.utils.timezone.now, editable=False, verbose_name='Последнее обновление')),
                ('checksum', isc_common.fields.code_field.CodeField(unique=True, verbose_name='Checksum MD5')),
                ('message', isc_common.fields.description_field.DescriptionField(verbose_name='Тело сообщения')),
                ('date_create', models.DateTimeField(db_index=True, default=django.utils.timezone.now, verbose_name='Дата записи')),
                ('isFolder', models.BooleanField(default=False)),
            ],
            options={
                'verbose_name': 'Сообщения',
                'db_table': 'tracker_messages_view',
                'managed': False,
            },
        ),
    ]



from django.urls import path

from isc_common.auth.views import usergroup_permission

urlpatterns = [

    path('Usergroup_permission/Fetch/', usergroup_permission.Usergroup_permission_Fetch),
    path('Usergroup_permission/Add', usergroup_permission.Usergroup_permission_Add),
    path('Usergroup_permission/Update', usergroup_permission.Usergroup_permission_Update),
    path('Usergroup_permission/Remove', usergroup_permission.Usergroup_permission_Remove),
    path('Usergroup_permission/Lookup', usergroup_permission.Usergroup_permission_Lookup),

]



import logging

from django.db.models import TextField, Model

logger = logging.getLogger(__name__)


class Debug(Model):
    params = TextField(null=True, blank=True)
    text = TextField()

    def __str__(self):
        return f'ID:{self.id}'

    def __repr__(self):
        return self.__str__()

    class Meta:
        verbose_name = 'Отладочная информация'



import logging

from django.conf import settings
from django.core.management import BaseCommand

from isc_common.auth.models.user import User
from isc_common.ws.progressStack import ProgressSerialStack

logger = logging.getLogger(__name__)


class Command(BaseCommand):

    # def add_arguments(self, parser):
    #     parser.add_argument('--mode', type=str)
    #     parser.add_argument('--qty', type=int)
    #     parser.add_argument('--id_user', type=int)

    def handle(self, *args, **options):
        user = User.objects.get(id=2)

        def func(progress):
            qty = 10
            print(user)
            for i in range(qty):
                progress.show(title=f'Тест {i}', label_contents=f'Text {i}', id=i)

            for i in range(qty):
                for j in range(qty + 1):
                    progress.setPercentsDone(percent=j * 10, id=i)

            for i in range(qty):
                progress.closed(id=i)

        ProgressSerialStack(
            func=func,
            host=settings.WS_HOST,
            port=settings.WS_PORT,
            channel=f'common_{user.username}',
        )



# Generated by Django 3.0.3 on 2020-02-09 17:41

from django.db import migrations, models
import uuid


class Migration(migrations.Migration):

    dependencies = [
        ('isc_common', '0043_auto_20200209_1728'),
    ]

    operations = [
        migrations.AlterField(
            model_name='progresses',
            name='guid',
            field=models.UUIDField(default=uuid.UUID('1972b80d-d298-4671-ae8c-df8c67e2e6b4')),
        ),
    ]



from django.urls import path

from history.views import visitor

urlpatterns = [

    path('Visitor/Fetch/', visitor.Visitor_Fetch),
    path('Visitor/Add', visitor.Visitor_Add),
    path('Visitor/Update', visitor.Visitor_Update),
    path('Visitor/Remove', visitor.Visitor_Remove),
    path('Visitor/Lookup/', visitor.Visitor_Lookup),
    path('Visitor/Info/', visitor.Visitor_Info),
    path('Visitor/Copy', visitor.Visitor_Copy),

]



import logging

from django.conf import settings
from django.utils.deprecation import MiddlewareMixin

from history import utils
from history.models.history import History
from isc_common.http.DSResponse import Body
from isc_common.json import BytesToJson, StrToJson

logger = logging.getLogger(__name__)


class LoggingRequestMiddleware(MiddlewareMixin):
    def process_request(self, request):

        if request.content_type not in ['text/xml', 'application/json']:
            return

        self.json = BytesToJson(request.body)

        if settings.INCLUDE_REQUEST_PATHES is None:
            settings.INCLUDE_REQUEST_PATHES = set('Add', 'Update', 'Remove')

        if isinstance(settings.INCLUDE_REQUEST_PATHES, list):
            settings.INCLUDE_REQUEST_PATHES = set(settings.INCLUDE_REQUEST_PATHES)

        if settings.DEBUG:
            for param in ['Get', 'Fetch']:
                settings.INCLUDE_REQUEST_PATHES.add(param)

        if request.path != '/':
            user = Body(request.body, request.session).user
            if user:
                username = user.username
                fio = user.get_short_name
                if username is not None:
                    path = set(request.path.split('/'))
                    enable_logging = len(path.intersection(settings.INCLUDE_REQUEST_PATHES)) > 0
                    if enable_logging and len(self.json) > 0:
                        ip_address = utils.get_ip(request)
                        user_agent = request.META.get('HTTP_USER_AGENT')

                        history_element = History.objects.using('history').create(
                            username=username,
                            fio=fio,
                            method=request.method,
                            ip_address=ip_address,
                            user_agent=user_agent,
                            path=request.path,
                            data=self.json
                        )

                        # history_element.ip_address=ip()

                        logger.debug('=========================================================================================================')
                        logger.debug(f'date: {history_element.date}')
                        logger.debug(f'username: {username}')
                        logger.debug(f'method: {request.method}')
                        logger.debug(f'path: {request.path}')
                        logger.debug(f'ip_address: {ip_address}')
                        logger.debug(f'user_agent: {user_agent}')
                        logger.debug(f'json: {StrToJson(self.json)}')
                        logger.debug('=========================================================================================================\n')

    def process_response(self, request, response):
        return response



from isc_common.isc.data_binding.criterion import Criterion


class AdvancedCriteria(Criterion):
    _constructor = None
    criteria = []

    def get_criterion(self, field_name):
        if isinstance(self.criteria, list):
            return list(filter(lambda x: x.fieldName == field_name, self.criteria))
        else:
            return None

    def __init__(self, **kwargs):
        lst = kwargs.get('lst')
        if isinstance(lst, list):
            for l in lst:
                for k, v in l.items():
                    if isinstance(v, list):
                        setattr(self, k, [AdvancedCriteria(**criterion) if criterion.get('_constructor') == 'AdvancedCriteria' else Criterion(**criterion) for criterion in v])
                    else:
                        setattr(self, k, v() if callable(v) else v)

        else:
            for k, v in kwargs.items():
                if isinstance(v, list):
                    setattr(self, k, [AdvancedCriteria(**criterion) if criterion.get('_constructor') == 'AdvancedCriteria' else Criterion(**criterion) for criterion in v])
                else:
                    setattr(self, k, v() if callable(v) else v)



import logging

from django.db.models import TextField, FloatField, DateTimeField, IntegerField

from isc_common.fields.related import ForeignKeyProtect
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet
from refs.models.type_param import Type_param

logger = logging.getLogger(__name__)


class Type_param_valuesQuerySet(AuditQuerySet):
    def filter(self, *args, **kwargs):
        return super().filter(*args, **kwargs)


class Type_param_valuesManager(AuditManager):

    @classmethod
    def getRecord(cls, record):
        res = {
            'id': record.id,
            'type_id': record.type.id,
            'type__code': record.type.code,
            'type__name': record.type.name,
            'type__type': record.type.type,
            'type__pick_list': record.pick_list,
            'type__length': record.length,
            'type__precision': record.precision,
            'type__exponent': record.exponent,
            'value_str': record.value_str,
            'value_float': record.value_float,
            'value_int': record.value_int,
            'value_date': record.value_date,
        }
        return res

    def get_queryset(self):
        return Type_param_valuesQuerySet(self.model, using=self._db)


class Type_param_values(AuditModel):
    type = ForeignKeyProtect(Type_param)
    value_str = TextField(null=True, blank=True)
    value_float = FloatField(null=True, blank=True)
    value_int = IntegerField(null=True, blank=True)
    value_date = DateTimeField(null=True, blank=True)

    objects = Type_param_valuesManager()

    def __str__(self):
        return f"{self.id}"

    class Meta:
        verbose_name = 'Значение параметров'



# Generated by Django 2.2.5 on 2019-09-13 10:56

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('twits', '0016_auto_20190913_1055'),
    ]

    operations = [
        migrations.AlterField(
            model_name='chat_user_user',
            name='common_id',
            field=models.PositiveIntegerField(),
        ),
    ]



import logging

from django.core.management import BaseCommand

import code_generator

logger = logging.getLogger(__name__)
appModel = None


class Command(BaseCommand):
    help = "Создание приложения."

    def handle(self, *args, **options):
        try:
            logger.debug(self.help)

            model = code_generator.appModel()
            model.write_entity()

            logger.debug(f'{self.help}, завершено.')
        except Exception as ex:
            raise ex



from django.urls import path

from isc_common.views import debug

urlpatterns = [

    path('Debug/Fetch/', debug.Debug_Fetch),
    path('Debug/Add', debug.Debug_Add),
    path('Debug/Update', debug.Debug_Update),
    path('Debug/Remove', debug.Debug_Remove),
    path('Debug/Lookup/', debug.Debug_Lookup),
    path('Debug/Info/', debug.Debug_Info),
    path('Debug/Copy', debug.Debug_Copy),

]



import logging
import os
import shutil

from django.conf import settings
from django.core.management import BaseCommand

from code_generator.core.writer_model import WriterModel
from one_c.models.entities import Field
from one_c.models.entity_1c import Entity_1c

logger = logging.getLogger(__name__)


class Command(BaseCommand):
    help = "Создание мат вьюшек для документов из 1С"

    entities = settings.ONE_C_ENTITIES()

    def fill_space(self, qty):
        res = ''
        while True:
            res += ' '
            qty -= 1
            if qty == 0:
                return res

    def uncapitalize(self, _str):
        if not isinstance(_str, str):
            _str = str(_str)
        return _str[0:1].lower() + _str[1:]

    def capitalize(self, _str):
        if not isinstance(_str, str):
            _str = str(_str)
        return _str[0:1].upper() + _str[1:]

    def dbl_qutes_str(self, str):
        return f'"{str}"'

    def qutes_str(self, str):
        return f"'{str}'"

    def handle(self, *args, **options):
        try:
            if not os.path.exists(settings.ONE_C_DIR):
                raise Exception(f'Path: {settings.ONE_C_DIR} not exists.')

            if not os.path.exists(settings.BASE_PATH_ONEC_TEMPLATE):
                raise Exception(f'Path: {settings.BASE_PATH_ONEC_TEMPLATE} not exists.')

            group_replace_map = dict(
                ITEMS=[],
                GROUP_NAME="OneC",
                ICON="Common.ur_entity",
                TITLE="Пользователи",
                TITLE_BUTTON="1С",
            )

            isc_replace_map = dict(
                ITEMD_EDITOR=[]
            )

            common_urls_file = f'{settings.ONE_C_DIR}{os.sep}common_urls.py'
            writerCommonUrls = WriterModel(path_output=common_urls_file, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}common_urls.pmd')

            isc_fields = isc_replace_map.get('ITEMD_EDITOR')
            isc_fields.append(f'{self.fill_space(2)}readonly {group_replace_map.get("GROUP_NAME")} : {group_replace_map.get("GROUP_NAME")}Statics;')
            isc_output_file = f'{settings.ONE_C_DIR}{os.sep}ts{os.sep}Isc.ts'
            writerIsc = WriterModel(path_output=isc_output_file, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}Isc.pmd')

            group_items = group_replace_map.get('ITEMS')
            group_output_file = f'{settings.ONE_C_DIR}{os.sep}ts{os.sep}groups{os.sep}group_one_c.ts'
            writerGroup = WriterModel(path_output=group_output_file, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}group.pmd')

            groups_output_dir = f'{settings.ONE_C_DIR}{os.sep}ts{os.sep}groups'
            if os.path.exists(groups_output_dir):
                shutil.rmtree(groups_output_dir)
            os.makedirs(groups_output_dir)

            model_output_dir = f'{settings.ONE_C_DIR}{os.sep}models'
            if os.path.exists(model_output_dir):
                shutil.rmtree(model_output_dir)
            os.makedirs(model_output_dir)

            views_output_dir = f'{settings.ONE_C_DIR}{os.sep}views'
            if os.path.exists(views_output_dir):
                shutil.rmtree(views_output_dir)
            os.makedirs(views_output_dir)

            urls_output_dir = f'{settings.ONE_C_DIR}{os.sep}urls'
            if os.path.exists(urls_output_dir):
                shutil.rmtree(urls_output_dir)
            os.makedirs(urls_output_dir)

            datasource_output_dir = f'{settings.ONE_C_DIR}{os.sep}ts{os.sep}datasources'
            if os.path.exists(datasource_output_dir):
                shutil.rmtree(datasource_output_dir)
            os.makedirs(datasource_output_dir)

            list_grid_editor_output_dir = f'{settings.ONE_C_DIR}{os.sep}ts{os.sep}editors'
            if os.path.exists(list_grid_editor_output_dir):
                shutil.rmtree(list_grid_editor_output_dir)
            os.makedirs(list_grid_editor_output_dir)

            menu_item_output_dir = f'{settings.ONE_C_DIR}{os.sep}ts{os.sep}menu_items'
            if os.path.exists(menu_item_output_dir):
                shutil.rmtree(menu_item_output_dir)
            os.makedirs(menu_item_output_dir)

            lookup_fields_output_dir = f'{settings.ONE_C_DIR}{os.sep}ts{os.sep}lookup_fields'
            if os.path.exists(lookup_fields_output_dir):
                shutil.rmtree(lookup_fields_output_dir)
            os.makedirs(lookup_fields_output_dir)

            common_urls_map = dict(
                LIST_PATH=[]
            )
            common_urls = common_urls_map.get('LIST_PATH')

            for entity in sorted(Entity_1c.objects.filter(), key=lambda record: record.code):

                model_name = f'{entity.translited_code}_model'
                model_caption = f'"{entity.code}"'
                common_urls.append(f'{self.fill_space(4)}path({self.qutes_str("logic/")}, include({self.qutes_str("kaf_pas.k_one_c.urls." + self.uncapitalize(entity.translited_code))})),')

                model_output_file = f'{model_output_dir}{os.sep}{self.uncapitalize(entity.translited_code)}.py'
                views_output_file = f'{views_output_dir}{os.sep}{self.uncapitalize(entity.translited_code)}.py'
                urls_output_file = f'{urls_output_dir}{os.sep}{self.uncapitalize(entity.translited_code)}.py'

                datasource_output_file = f'{datasource_output_dir}{os.sep}{self.uncapitalize(entity.translited_code)}.ts'
                list_grid_editor_output_file = f'{list_grid_editor_output_dir}{os.sep}{self.uncapitalize(entity.translited_code)}.ts'
                menu_item_output_file = f'{menu_item_output_dir}{os.sep}{self.uncapitalize(entity.translited_code)}.ts'
                lookup_fields_output_file = f'{lookup_fields_output_dir}'
                lookup_fields_output_file_ext = '.ts'

                writerModel = WriterModel(path_output=model_output_file, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}model.pmd')
                writerViews = WriterModel(path_output=views_output_file, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}views.pmd')
                writerUrls = WriterModel(path_output=urls_output_file, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}urls.pmd')

                writerDatasource = WriterModel(path_output=datasource_output_file, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}datasource.pmd')
                writerTreeEditor = WriterModel(path_output=list_grid_editor_output_file, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}tree_grid_editor.pmd')
                writerMenuItem = WriterModel(path_output=menu_item_output_file, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}menu_item.pmd')
                writerLookUpItem = WriterModel(path_output=lookup_fields_output_file, output_file_ext=lookup_fields_output_file_ext, template_path=f'{settings.BASE_PATH_ONEC_TEMPLATE}{os.sep}templates{os.sep}look_up_field.pmd')

                field_mapping = [
                    ('entity_id', 'record.entity.id'),
                    ('entity__code', 'record.entity.code'),
                ]

                field_mapping.extend([((self.uncapitalize(item.translited_alias), f'record.{self.uncapitalize(item.translited_record_path)}')) for item in self.entities.get_fields(entity.id)])

                # Не передвигать !!!
                model_replace_map = dict(
                    QUERY_SET='''def get_range_rows1(self, request, function=None):
        request = DSRequest(request=request)
        self.alive_only = False
        return self.get_range_rows(start=request.startRow, end=request.endRow, function=function, json=request.json)''',
                    NAME_MODEL=model_name,
                    FIELDS_MAPPING=f'dict({",".join([f"{item[0]}={item[1]}" for item in field_mapping])})',
                    FIELDS=[
                        f'{self.fill_space(4)}entity = ForeignKeyProtect(Entity_1c)',
                    ],
                    IMPORTS=[
                        'from one_c.models.base_model import Base_model',
                        'from django.db.models import UUIDField, TextField, IntegerField, FloatField, BooleanField, BigIntegerField',
                        'from isc_common.fields.related import ForeignKeyProtect',
                        'from one_c.models.entity_1c import Entity_1c',
                        'from isc_common.http.DSRequest import DSRequest',
                    ],
                    _STR_='f"(ref: {self.ref}, entity: {self.entity})"',
                    META=[
                        f'{self.fill_space(8)}managed = False',
                        f'{self.fill_space(8)}db_table = "{entity.translited_code}_mview"'
                    ]
                )

                view_replace_map = dict(
                    PACKAGE_MODEL_MANAGER=f'from kaf_pas.k_one_c.models.{self.uncapitalize(entity.translited_code)} import {entity.translited_code}_model, {entity.translited_code}_modelManager',
                    NAME_MODEL=model_name,
                )

                url_replace_map = dict(
                    PACKAGE_MODEL=f'from kaf_pas.k_one_c.views import {self.uncapitalize(entity.translited_code)}',
                    PACKAGE_VIEW=entity.translited_code,
                    LPACKAGE_VIEW=self.uncapitalize(entity.translited_code),
                    NAME_MODEL=model_name,
                )

                datasource_replace_map = dict(
                    NAME_MODEL=model_name,
                    NAME_PARAM=entity.translited_code,
                    FIELDS=[]
                )

                tree_grid_editor_replace_map = dict(
                    NAME_MODEL=model_name,
                    NAME_CAPTION=model_caption,
                    ITEMS_TYPE=[
                        'new MiTreeRefresh(),'
                        'new MiTreeCopyValue(),'
                    ],
                    parentIdField='parent',
                    idField='ref',
                    PERMISSIONS=''
                )

                menu_item_replace_map = dict(
                    NAME_MODEL=model_name,
                    NAME_PARAM=entity.code,
                    ICON="Common.ur_entity"
                )

                isc_fields.append(f'{self.fill_space(2)}readonly {model_name} : {model_name}Statics;')

                group_items.append(f'{self.fill_space(48)}get_{model_name}(ownerApp),')
                # group_items.append(f'{self.fill_space(56)}<MenuSSItem>{{isSeparator: true}},')

                model_fields = model_replace_map.get('FIELDS')
                datasource_fields = datasource_replace_map.get('FIELDS')

                for used_field in self.entities.get_fields(entity.id):
                    if used_field:
                        if not used_field.only_datasource:
                            if used_field.foreign_key:
                                model_fields.append(f'{self.fill_space(4)}{self.uncapitalize(used_field.translited_name)} = {str(used_field.foreign_key)}')
                            elif used_field.type == Field.string():
                                model_fields.append(f'{self.fill_space(4)}{self.uncapitalize(used_field.translited_name)} = TextField(primary_key={self.capitalize(used_field.primary_key)})')
                            elif used_field.type == Field.uuid():
                                model_fields.append(f'{self.fill_space(4)}{self.uncapitalize(used_field.translited_name)} = UUIDField(primary_key={self.capitalize(used_field.primary_key)})')
                            elif used_field.type == Field.boolean():
                                model_fields.append(f'{self.fill_space(4)}{self.uncapitalize(used_field.translited_name)} = BooleanField()')
                            elif used_field.type == Field.int():
                                model_fields.append(f'{self.fill_space(4)}{self.uncapitalize(used_field.translited_name)} = IntegerField(primary_key={self.capitalize(used_field.primary_key)})')
                            elif used_field.type == Field.float():
                                model_fields.append(f'{self.fill_space(4)}{self.uncapitalize(used_field.translited_name)} = FloatField()')
                            else:
                                raise Exception(f'{used_field.type} is unknown type.')

                        if used_field.type == Field.boolean():
                            datasource_fields.append(f'{self.fill_space(12)}{"booleanField"}(' +
                                                     f'"{self.uncapitalize(used_field.translited_alias)}", ' +
                                                     f'"{used_field.title}", ' +
                                                     f'{self.uncapitalize(used_field.required)},' +
                                                     f'{self.uncapitalize(used_field.hidden)},' +
                                                     f'{self.uncapitalize(used_field.canEdit)},' +
                                                     f'{self.uncapitalize(used_field.canFilter)},' +
                                                     f'{self.uncapitalize(used_field.canGroupBy)},' +
                                                     f'{used_field.index}),')
                        elif used_field.type == Field.uuid():
                            datasource_fields.append(f'{self.fill_space(12)}{"uuidField"}(' +
                                                     f'"{self.uncapitalize(used_field.translited_alias)}", ' +
                                                     f'"{used_field.title}", ' +
                                                     f'{self.uncapitalize(used_field.required)},' +
                                                     f'{self.uncapitalize(used_field.primary_key)},' +
                                                     f'{used_field.length[0]},' +
                                                     f'{self.uncapitalize(used_field.hidden)},' +
                                                     f'{self.uncapitalize(used_field.canEdit)},' +
                                                     f'{self.uncapitalize(used_field.canFilter)},' +
                                                     f'{used_field.align},' +
                                                     f'{self.uncapitalize(used_field.canGroupBy)},' +
                                                     f'{used_field.index}),')
                        elif used_field.type == Field.string():
                            datasource_fields.append(f'{self.fill_space(12)}{"nameField"}(' +
                                                     f'"{self.uncapitalize(used_field.translited_alias)}", ' +
                                                     f'"{used_field.title}", ' +
                                                     f'{self.uncapitalize(used_field.required)},' +
                                                     f'{self.uncapitalize(used_field.primary_key)},'
                                                     f'{used_field.length[0]},' +
                                                     f'{self.uncapitalize(used_field.hidden)},' +
                                                     f'{self.uncapitalize(used_field.canEdit)},' +
                                                     f'{self.uncapitalize(used_field.canFilter)},' +
                                                     f'{used_field.align},' +
                                                     f'{self.uncapitalize(used_field.canSort)},' +
                                                     f'{self.uncapitalize(used_field.canGroupBy)},' +
                                                     f'{used_field.index}),')
                        elif used_field.type == Field.float():
                            datasource_fields.append(f'{self.fill_space(12)}{"floatField"}(' +
                                                     f'"{self.uncapitalize(used_field.translited_alias)}", ' +
                                                     f'"{used_field.title}", ' +
                                                     f'{self.uncapitalize(used_field.required)},' +
                                                     f'{self.uncapitalize(used_field.primary_key)},'
                                                     f'{self.uncapitalize(used_field.hidden)},' +
                                                     f'{self.uncapitalize(used_field.canEdit)},' +
                                                     f'{self.uncapitalize(used_field.canFilter)},' +
                                                     f'{used_field.align},' +
                                                     f'{self.uncapitalize(used_field.format)},' +
                                                     f'{self.uncapitalize(used_field.canGroupBy)},' +
                                                     f'{used_field.index}),')
                        elif used_field.type == Field.int():
                            datasource_fields.append(f'{self.fill_space(12)}{"integerField"}(' +
                                                     f'"{self.uncapitalize(used_field.translited_alias)}", ' +
                                                     f'"{used_field.title}", ' +
                                                     f'{self.uncapitalize(used_field.required)},' +
                                                     f'{self.uncapitalize(used_field.primary_key)},'
                                                     f'{self.uncapitalize(used_field.hidden)},' +
                                                     f'{self.uncapitalize(used_field.canEdit)},' +
                                                     f'{self.uncapitalize(used_field.canFilter)},' +
                                                     f'{used_field.align},' +
                                                     f'{self.uncapitalize(used_field.format)},' +
                                                     f'{self.uncapitalize(used_field.canGroupBy)}),')

                        if used_field.lookup:
                            lookup_fields_replace_map = dict(
                                NAME_MODEL=model_name,
                                U_NAME_MODEL=self.uncapitalize(model_name),
                                NAME_PARAM=self.uncapitalize(used_field.translited_name),
                                TITLE=used_field.title,
                            )

                            writerLookUpItem.output_file_name = f'{self.uncapitalize(entity.translited_code)}_{self.uncapitalize(used_field.translited_name)}'
                            writerLookUpItem.write_entity(replace_map=lookup_fields_replace_map)

                logger.debug(f'Write: {model_output_file}')

                writerCommonUrls.write_entity(replace_map=common_urls_map)
                writerModel.write_entity(replace_map=model_replace_map)
                writerViews.write_entity(replace_map=view_replace_map)
                writerUrls.write_entity(replace_map=url_replace_map)
                writerDatasource.write_entity(replace_map=datasource_replace_map)
                writerTreeEditor.write_entity(replace_map=tree_grid_editor_replace_map)
                writerMenuItem.write_entity(replace_map=menu_item_replace_map)

            writerIsc.write_entity(replace_map=isc_replace_map)
            writerGroup.write_entity(replace_map=group_replace_map)

        except Exception as ex:
            raise ex



# Generated by Django 2.2.4 on 2019-08-25 10:24

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('clndr', '0062_auto_20190825_1019'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='calendar_shifts_days',
            options={'ordering': ['shiftorder'], 'verbose_name': 'Рабочие смены по дням'},
        ),
        migrations.RemoveField(
            model_name='shift_day',
            name='numdayinweek',
        ),
    ]



import logging

from django.db.models import Model

from isc_common import setAttr, delAttr
from isc_common.bit import TurnBitOn, TurnBitOff
from isc_common.fields.code_field import CodeField
from isc_common.fields.description_field import DescriptionField
from isc_common.fields.name_field import NameField
from isc_common.fields.related import ForeignKeyProtect
from isc_common.http.DSRequest import DSRequest
from isc_common.models.audit import AuditModel, AuditManager, AuditQuerySet

logger = logging.getLogger(__name__)


class BaseRefQuerySet(AuditQuerySet):

    def filter(self, *args, **kwargs):
        # if self.alive_only:
        #     setAttr(kwargs, 'deleted_at__isnull', True)

        key = [key for key in kwargs.keys() if key == 'full_name']
        if len(key) == 0:
            return super().filter(*args, **kwargs)
        else:
            full_name = kwargs.get('full_name')
            if full_name is not None:
                res = self.get_full_name_id(None, full_name[1:])
                delAttr(kwargs, 'full_name')
                setAttr(kwargs, 'pk', res.pk)
            else:
                setAttr(kwargs, 'pk', 0)
            return super().filter(*args, **kwargs)

    def get_full_name_id(self, parent_id, name):
        end = False
        if name.find('/') != -1:
            _name = name[:name.find('/')]
        else:
            end = True
            _name = name

        if parent_id:
            res = super().get(parent_id=parent_id, name=_name)
        else:
            res = super().get(parent__isnull=True, name=_name)

        if end:
            return res
        else:
            return self.get_full_name_id(res.id, name[name.find('/') + 1:])

    def get(self, *args, **kwargs):
        key = [key for key in kwargs.keys() if key == 'full_name']
        if len(key) == 0:
            res = super().get(*args, **kwargs)
        else:
            full_name = kwargs.get('full_name')
            if full_name is None:
                full_name = ''
            res = self.get_full_name_id(None, full_name[1:])

        return res

    def update(self, **kwargs):
        if kwargs.get('code') is None and kwargs.get('name') is not None:
            setAttr(kwargs, 'code', AuditModel.translit(kwargs.get('name')))
        res = super().update(**kwargs)
        return res

    def create(self, **kwargs):
        if kwargs.get('code') is None and kwargs.get('name') is not None:
            setAttr(kwargs, 'code', AuditModel.translit(kwargs.get('name')))
        res = super().create(**kwargs)
        return res


class BaseRefManager(AuditManager):
    def get_queryset(self):
        if self.alive_only:
            return BaseRefQuerySet(model=self.model, alive_only=self.alive_only).filter(deleted_at=None)
        return BaseRefQuerySet(model=self.model, alive_only=False)


class StatusBaseRefManager(BaseRefManager):
    def createFromRequest(self, request):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        disabled = _data.get('disabled')
        props = _data.get('props', 0)

        delAttr(_data, 'disabled')
        props = TurnBitOn(props, 0) if disabled else TurnBitOff(props, 0)
        setAttr(_data, 'props', props)

        super().create(**_data)
        setAttr(data, 'props', props)
        return data

    def updateFromRequest(self, request, removed=None, function=None):
        request = DSRequest(request=request)
        data = request.get_data()
        _data = data.copy()

        _data = self.delete_underscore_element(data=_data)
        disabled = _data.get('disabled')
        props = _data.get('props', 0)

        delAttr(_data, 'disabled')
        delAttr(_data, 'id')
        props = TurnBitOn(props, 0) if disabled else TurnBitOff(props, 0)
        setAttr(_data, 'props', props)
        setAttr(data, 'props', props)

        super().filter(id=data.get('id')).update(**_data)
        return data


class BaseRef(AuditModel):
    code = CodeField()
    name = NameField()
    description = DescriptionField()

    def save(self, *args, **kwargs):
        if self.code is None:
            self.code = AuditModel.translit(self.name)
        super().save(*args, **kwargs)

    class Meta:
        abstract = True


class BaseHierarcy(Model):
    code = None
    name = None

    parent = ForeignKeyProtect("self", null=True, blank=True)

    def get_parent_name(self, parent, name):
        if not self.name:
            raise Exception('Name not Exists')

        if not parent:
            return name

        item = parent
        name = f'/{item.name}{name}'
        if item.parent:
            parent = self.get_parent_name(item.parent, name)
            return parent
        return name

    def get_root(self, parent):

        if not parent:
            return self

        if parent.parent:
            return self.get_parent_name(parent.parent)
        return parent

    @property
    def full_name(self):
        if not self.name:
            if self.code:
                self.name = self.code
            else:
                raise Exception('Name and Code not Exists')

        name = f'/{self.name}'
        return self.get_parent_name(self.parent, name)

    @property
    def full_name_id(self):
        if not self.code:
            return None

        return self.objects.get(full_name=self.full_name)

    @property
    def root(self):
        return self.get_root(self.parent)

    objects = BaseRefManager()
    all_objects = BaseRefManager(alive_only=False)

    class Meta:
        abstract = True


class Hierarcy(AuditModel, BaseHierarcy):
    class Meta:
        abstract = True


class BaseRefHierarcy(Hierarcy):
    code = CodeField(db_index=True)
    name = NameField()
    description = DescriptionField()

    def save(self, *args, **kwargs):
        if self.code is None:
            self.code = AuditModel.translit(self.name)
        super().save(*args, **kwargs)

    class Meta:
        abstract = True
